@@ -1,23 +1,28 @@
 package WWW::Challonge;
 use WWW::Challonge::Tournament;
-use REST::Client;
+use LWP::UserAgent;
+use Carp qw/carp croak/;
 use JSON qw/to_json from_json/;
 
-use 5.006;
+use 5.010;
 use strict;
 use warnings;
 
+sub __handle_error;
+sub __json_request;
+
 =head1 NAME
 
 WWW::Challonge - Perl wrapper for the Challonge API
 
 =head1 VERSION
 
-Version 0.20
+Version 0.31
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.31';
+our $HOST = "https://api.challonge.com/v1";
 
 =head1 SYNOPSIS
 
@@ -33,34 +38,57 @@ as documented L<here|http:://api.challonge.com/v1>.
 
 =head2 new
 
-Creates a new C<WWW::Challonge> object. Takes in an API key, which is required:
+Creates a new C<WWW::Challonge> object. Takes in an API key, which is required,
+and optionally a preconfigured L<LWP::UserAgent> object, which is mostly used for
+testing.
 
-    my $c = WWW::Challonge->new($api_key);
+    my $c  = WWW::Challonge->new($api_key);
+	my $c2 = WWW::Challonge->new({ key => $api_key, client => $my_client });
 
 =cut
 
 sub new
 {
-	# Get the API key:
+	# Get the arguments:
 	my $class = shift;
-	my $key = shift;
+	my $args = shift;
 
-	# Create a REST client to interface Challonge:
-	my $client = REST::Client->new();
-	$client->setHost("https://api.challonge.com/v1");
+	my $key;
+	my $client;
 
-	# Try to get some content and check the response code:
-	$client->GET("/tournaments.json?api_key=$key");
-
-	# Check to see if the API key is valid:
-	if($client->responseCode() eq '401')
+	# If the argument is a scalar, use it as the API key:
+	if(! ref $args)
 	{
-		# If it isn't, warn the user and exit:
-		print STDERR "Error: Challonge API key is invalid.\n";
-		return undef;
+		$key = $args;
+
+		# Create an LWP useragent to interface Challonge:
+		$client = LWP::UserAgent->new;
+
+		# Try to get some content and check the response code:
+		my $response = $client->get("$HOST/tournaments.json?api_key=$key");
+
+		# Check to see if the API key is valid:
+		if($response->is_error)
+		{
+			croak "Challonge API key is invalid" if($response->code == 401);
+			__handle_error $response;
+		}
+	}
+	elsif(ref $args eq "HASH")
+	{
+		croak "'client' must be a LWP::UserAgent object"
+			unless((defined $args->{client}) &&
+			(UNIVERSAL::isa($args->{client}, "LWP::UserAgent")));
+
+		$client = $args->{client};
+		$key = $args->{key} // "";
+	}
+	else
+	{
+		croak "Expected scalar or hashref";
 	}
 
-	# Otherwise, keep the key and the client in an object and return:
+	# Create and return the object:
 	my $c = { key => $key, client => $client };
 	bless $c, $class;
 }
@@ -147,7 +175,7 @@ sub index
 	my $client = $self->{client};
 
 	# The intial request URL:
-	my $req = "/tournaments.json?api_key=$key";
+	my $req = "$HOST/tournaments.json?api_key=$key";
 
 	# Loop through the options (if any) and add them on:
 	for my $option(keys %{$options})
@@ -157,37 +185,37 @@ sub index
 		{
 			if($options->{$option} !~ /^all|pending|in_progress|ended$/)
 			{
-				print STDERR "Error: Argument '" . $options->{option} .
-					"' for option '$option' is invalid.";
+				croak "Argument '" . $options->{option} .
+					"' for option '$option' is invalid";
 			}
 		}
 		elsif($option =~ /^type$/)
 		{
 			if($options->{$option} !~ /^(single|double)_elimination|round_robin|swiss$/)
 			{
-				print STDERR "Error: Argument '" . $options->{option} .
-					"' for option '$option' is invalid.";
+				croak "Argument '" . $options->{option} .
+					"' for option '$option' is invalid";
 			}
 		}
 		elsif($option =~ /^created_(before|after)$/)
 		{
 			if($options->{$option} !~ /^\d{4}-\d{2}-\d{2}$/)
 			{
-				print STDERR "Error: Argument '" . $options->{option} .
-					"' for option '$option' is invalid.";
+				croak "Argument '" . $options->{option} .
+					"' for option '$option' is invalid";
 			}
 		}
 		elsif($option =~ /^subdomain$/)
 		{
 			if($options->{$option} !~ /^[a-zA-Z0-9_]*$/)
 			{
-				print STDERR "Error: Argument '" . $options->{option} .
-					"' for option '$option' is invalid.";
+				croak "Argument '" . $options->{option} .
+					"' for option '$option' is invalid";
 			}
 		}
 		else
 		{
-			print STDERR "Error: Option '$option' is invalid.";
+			carp "Unknown option '$option'";
 			return undef;
 		}
 
@@ -195,11 +223,14 @@ sub index
 	}
 
 	# Make the request:
-	$client->GET($req);
+	my $response = $client->get($req);
+
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# Make a new tournament object for every tourney returned:
 	my @tournaments;
-	for my $tournament(@{from_json($client->responseContent())})
+	for my $tournament(@{from_json($response->decoded_content)})
 	{
 		push @tournaments, WWW::Challonge::Tournament->new($tournament,
 			$key, $client);
@@ -227,23 +258,19 @@ sub show
 	my $self = shift;
 	my $url = shift;
 
-	# Get the key and REST client:
+	# Get the key and client:
 	my $key = $self->{key};
 	my $client = $self->{client};
 
 	# Try to get the tournament:
-	$client->GET("/tournaments/$url.json?api_key=$key");
+	my $response = $client->get("$HOST/tournaments/$url.json?api_key=$key");
 
 	# Check for any errors:
-	if($client->responseCode eq '404')
-	{
-		print STDERR "Error: Tournament '$url' not found.\n";
-		return undef;
-	}
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# Otherwise create a tourney with the object and return it:
 	my $tourney = WWW::Challonge::Tournament->new(
-		from_json($client->responseContent), $key, $client);
+		from_json($response->decoded_content), $key, $client);
 	return $tourney;
 }
 
@@ -434,45 +461,81 @@ sub create
 	my $self = shift;
 	my $args = shift;
 
-	# Get the key and REST client:
+	# Get the key and client:
 	my $key = $self->{key};
 	my $client = $self->{client};
 
 	# Fail if name and URL aren't given:
 	if((! defined $args->{name}) && (! defined $args->{url}))
 	{
-		print STDERR "Error: Name and URL are required to create a ",
-			"tournament.\n";
-		return undef;
+		croak "Name and URL are required to create a tournament";
 	}
 
 	# Check the arguments and values are valid:
 	return undef unless(WWW::Challonge::Tournament::__args_are_valid($args));
 
-	# Add in the API key and convert to a POST request:
+	# Encapsulate the API key and the arguments in a single hashref:
 	my $params = { api_key => $key, tournament => $args };
 
 	# Now we have all the arguments validated, send the POST request:
-	$client->POST("/tournaments.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(
+		__json_request("$HOST/tournaments.json", "POST", $params));
 
 	# Check for any errors:
-	if($client->responseCode >= 300)
-	{
-		my $error = from_json($client->responseContent)->{errors}->[0];
-		if($error =~ /taken/)
-		{
-			print STDERR "Error: URL '", $args->{url}, "' is already taken\n";
-		}
-		return undef;
-	}
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# Otherwise, make a tournament object and return it:
 	my $t = WWW::Challonge::Tournament->new(
-		from_json($client->responseContent), $key, $client);
+		from_json($response->decoded_content), $key, $client);
 	return $t;
 }
 
+=head2 __handle_error
+
+Used throughout the module to deal with when the Challonge API returns an
+error of some kind. Takes a L<HTTP::Response> object.
+
+=cut
+
+sub __handle_error
+{
+	my $response = shift;
+
+	# Determine if the response is JSON or not:
+	eval { from_json($response->decoded_content) };
+	unless($@)
+	{
+		for my $error(@{from_json($response->decoded_content)->{errors}})
+		{
+			croak "(" . $response->code . ") " . $error;
+		}
+	}
+	else
+	{
+		croak $response->code;
+	}
+}
+
+=head2 __json_request
+
+Transforms a URI, verb and hashref to a L<HTTP::Request> object to use for a
+POST/PUT request with the hashref transformed to JSON.
+
+=cut
+
+sub __json_request
+{
+	my $uri = shift;
+	my $action = shift;
+	my $params = shift;
+
+	my $req = HTTP::Request->new($action, $uri);
+	$req->header('Content-Type' => 'application/json');
+	$req->content(to_json($params));
+
+	return $req;
+}
+
 =head1 AUTHOR
 
 Alex Kerr, C<< <kirby at cpan.org> >>
