@@ -1,9 +1,11 @@
 package WWW::Challonge::Match;
 
-use 5.006;
+use 5.010;
 use strict;
 use warnings;
+use WWW::Challonge;
 use WWW::Challonge::Match::Attachment;
+use Carp qw/carp croak/;
 use JSON qw/to_json from_json/;
 
 sub __args_are_valid;
@@ -15,11 +17,11 @@ a Challonge tournament.
 
 =head1 VERSION
 
-Version 0.20
+Version 0.31
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.31';
 
 =head1 SUBROUTINES/METHODS
 
@@ -94,6 +96,7 @@ sub update
 	my $client = $self->{client};
 	my $url = $self->{match}->{tournament_id};
 	my $id = $self->{match}->{id};
+	my $HOST = $WWW::Challonge::HOST;
 
 	my $params = { api_key => $key, match => { } };
 
@@ -104,8 +107,7 @@ sub update
 		if((ref $args eq "HASH") && ((! defined $args->{scores_csv}) ||
 			(ref $args->{scores_csv} ne "ARRAY")))
 		{
-			print STDERR "Error: Required argument 'scores_csv' as an array ",
-				"reference of scores.\n";
+			carp "Required argument 'scores_csv' as an array reference";
 			return undef;
 		}
 
@@ -151,27 +153,18 @@ sub update
 		}
 
 		# Make the PUT call:
-		$client->PUT("/tournaments/$url/matches/$id.json", to_json($params),
-			{ "Content-Type" => 'application/json' });
+		my $response = $client->request(WWW::Challonge::__json_request(
+			"$HOST/tournaments/$url/matches/$id.json", "PUT", $params));
 
-		# Check if it was successful:
-		if($client->responseCode > 300)
-		{
-			my $errors = from_json($client->responseContent)->{errors};
-			for my $error(@{$errors})
-			{
-				print STDERR "Error: $error\n";
-			}
-			return undef;
-		}
+		# Check for any errors:
+		WWW::Challonge::__handle_error $response if($response->is_error);
 
-		# If so, update the store of the match:
-		$self->{match} = from_json($client->responseContent)->{match};
+		return 1;
 	}
 	else
 	{
 		# Otherwise, give an error and exit:
-		print STDERR "Error: Expected an arrayref or hashref.\n";
+		croak "Expected an arrayref or hashref";
 		return undef;
 	}
 }
@@ -251,23 +244,17 @@ sub attributes
 	my $client = $self->{client};
 	my $url = $self->{match}->{tournament_id};
 	my $id = $self->{match}->{id};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Get the most recent version:
-	$client->GET("/tournaments/$url/matches/$id.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/matches/$id.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, save it and then return it:
-	$self->{match} = from_json($client->responseContent)->{match};
+	# If not, save it and then return it:
+	$self->{match} = from_json($response->decoded_content)->{match};
 	return $self->{match};
 }
 
@@ -289,24 +276,18 @@ sub index
 	my $client = $self->{client};
 	my $url = $self->{match}->{tournament_id};
 	my $id = $self->{match}->{id};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Get the match attachments:
-	$client->GET("/tournaments/$url/matches/$id/attachments.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/matches/$id/attachments.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If it was successful, create the objects and return them:
 	my $attachments = [];
-	for my $att(@{from_json($client->responseContent)})
+	for my $att(@{from_json($response->decoded_content)})
 	{
 		push @{$attachments},
 			WWW::Challonge::Match::Attachment->new($att, $url, $key, $client);
@@ -333,25 +314,18 @@ sub show
 	my $client = $self->{client};
 	my $url = $self->{match}->{tournament_id};
 	my $id = $self->{match}->{id};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Get the match attachments:
-	$client->GET(
-		"/tournaments/$url/matches/$id/attachments/$atth.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/matches/$id/attachments/$atth.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If it was successful, create the object and return it:
 	my $attachment = WWW::Challonge::Match::Attachment->new(
-		from_json($client->responseContent),
+		from_json($response->decoded_content),
 		$url,
 		$key,
 		$client
@@ -370,8 +344,7 @@ must be true for this to succeed.
 
 =item asset
 
-Currently unsupported. A file upload (250KB). If provided, the 'url' parameter
-will be ignored.
+A file upload (max 250KB). If provided, the 'url' parameter will be ignored.
 
 =item url
 
@@ -385,7 +358,7 @@ Text to the describte the file or URL, or it can simply be standalone text.
 
 	# A simple URL:
 	my $ma = $m->create({
-		url => www.example.com/image.png",
+		url => http://www.example.com/image.png",
 		description => "An example URL",
 	});
 
@@ -407,39 +380,30 @@ sub create
 	my $client = $self->{client};
 	my $url = $self->{match}->{tournament_id};
 	my $id = $self->{match}->{id};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Check the arguments are valid:
 	return undef
 		unless(WWW::Challonge::Match::Attachment::__args_are_valid($args));
 
-#	if(defined $args->{asset})
-#	{
-#		$args->{asset} = [ $args->{asset} ];
-#	}
+	# Wrap the filename in an arrayref for HTTP::Request::Common:
+	$args->{asset} = [ $args->{asset} ] if(defined $args->{asset});
 
 	# Make the POST call:
-	my $params = { api_key => $key, match_attachment => $args };
-#	$client->POST("/tournaments/$url/matches/$id/attachments.json",
-#		$params, { "Content-Type" => 'multipart/form-data' });
-	$client->POST("/tournaments/$url/matches/$id/attachments.json",
-		to_json($params), { "Content-Type" => 'application/json' });
+	my @params = map { "match_attachment[" . $_ . "]" => $args->{$_} }
+		keys %{$args};
+	my $response = $client->post(
+		"$HOST/tournaments/$url/matches/$id/attachments.json",
+		"Content-Type" => 'form-data',
+		"Content" => [ "api_key" => $key, @params ],
+	);
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		print $client->responseCode, "\n";
-		print $client->responseContent, "\n";
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If so, make an object and return it:
 	return WWW::Challonge::Match::Attachment->new(
-		from_json($client->responseContent),
+		from_json($response->decoded_content),
 		$url,
 		$key,
 		$client
@@ -463,8 +427,8 @@ sub __args_are_valid
 	{
 		if($result !~ /^\d*-\d*$/)
 		{
-			print STDERR "Error: Results must be given in the format \"x-y\",",
-				" where x and y are integers.\n";
+			carp "Results must be given in the format \"x-y\", where x and y ".
+			"are integers";
 			return undef;
 		}
 	}
@@ -477,8 +441,7 @@ sub __args_are_valid
 			next unless(defined $args->{$arg});
 			if($args->{$arg} !~ /^\d*$/)
 			{
-				print STDERR "Error: Argument '", $arg, "' must be an integer",
-					".\n";
+				carp "Argument '", $arg, "' must be an integer";
 				return undef;
 			}
 		}
@@ -496,8 +459,7 @@ sub __args_are_valid
 					last;
 				}
 			}
-			print STDERR "Warning: Ignoring unknown argument '", $arg, "'\n"
-				unless($is_valid);
+			carp "Ignoring unknown argument '$arg'" unless($is_valid);
 			$is_valid = 0;
 		}
 	}
