@@ -1,10 +1,11 @@
 package WWW::Challonge::Tournament;
 
-use 5.006;
+use 5.010;
 use strict;
 use warnings;
 use WWW::Challonge::Participant;
 use WWW::Challonge::Match;
+use Carp qw/carp croak/;
 use JSON qw/to_json from_json/;
 
 sub __is_kill;
@@ -16,11 +17,11 @@ WWW::Challonge::Tournament - A class representing a single Challonge tournament.
 
 =head1 VERSION
 
-Version 0.20
+Version 0.31
 
 =cut
 
-our $VERSION = '0.20';
+our $VERSION = '0.31';
 
 =head1 SUBROUTINES/METHODS
 
@@ -76,6 +77,7 @@ sub update
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Check the arguments and values are valid:
 	return undef unless(WWW::Challonge::Tournament::__args_are_valid($args));
@@ -84,22 +86,13 @@ sub update
 	my $params = { api_key => $key, tournament => $args };
 
 	# Make the PUT request:
-	$client->PUT("/tournaments/$url.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url.json", "PUT", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament} = from_json($client->responseContent)->{tournament};
+	return 1;
 }
 
 =head2 destroy
@@ -125,23 +118,18 @@ sub destroy
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Make the DELETE call:
-	$client->DELETE("/tournaments/$url.json?api_key=$key");
+	my $response = $client->delete("$HOST/tournaments/$url.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# Set the tournament to dead to prevent further operations:
+	# Set the tournament to dead:
 	$self->{alive} = 0;
+
+	return 1;
 }
 
 =head2 process_check_ins
@@ -180,27 +168,19 @@ sub process_check_ins
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Send the API key:
 	my $params = { api_key => $key };
 
 	# Make the POST call:
-	$client->POST("/tournaments/$url/process_check_ins.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/process_check_ins.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament}->{state} = "checked_in";
+	return 1;
 }
 
 =head2 abort_check_in
@@ -236,34 +216,26 @@ sub abort_check_in
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Send the API key:
 	my $params = { api_key => $key };
 
 	# Make the POST call:
-	$client->POST("/tournaments/$url/abort_check_in.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/abort_check_in.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament}->{state} = "pending";
+	return 1;
 }
 
 =head2 start
 
 Starts a tournament, opening up matches for score reporting. The tournament
-must have at least 2 participants. If successful, sets the status of the
-tournament to "in_progress".
+must have at least 2 participants. If successful, sets the state of the
+tournament to "underway".
 
 	$t->start;
 
@@ -280,33 +252,25 @@ sub start
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Send the API key:
 	my $params = { api_key => $key };
 
 	# Make the POST call:
-	$client->POST("/tournaments/$url/start.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/start.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament}->{state} = "in_progress";
+	return 1;
 }
 
 =head2 finalize
 
 Finalises a tournament that has had all match scores submitted, rendering the
-results permenant. If successful, it sets the status to "ended".
+results permenant. If successful, it sets the state to "complete".
 
 	$t->finalize;
 
@@ -323,33 +287,25 @@ sub finalize
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Send the API key:
 	my $params = { api_key => $key };
 
 	# Make the POST call:
-	$client->POST("/tournaments/$url/finalize.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/finalize.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament}->{state} = "ended";
+	return 1;
 }
 
 =head2 reset
 
 Resets an "in_progress" tournament, deleting all match records. You can add,
-remove or edit users before starting again.
+remove or edit users before starting again. Sets the state to "pending".
 
 	$t->reset;
 
@@ -366,27 +322,19 @@ sub reset
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Send the API key:
 	my $params = { api_key => $key };
 
 	# Make the POST call:
-	$client->POST("/tournaments/$url/reset.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/reset.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, update the object's store of the tournament:
-	$self->{tournament}->{state} = "ended";
+	return 1;
 }
 
 =head2 attributes
@@ -540,23 +488,18 @@ sub attributes
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Get the most recent version:
-	$client->GET("/tournaments/$url.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# Save the most recent version and return it:
-	$self->{tournament} = from_json($client->responseContent)->{tournament};
+	$self->{tournament} = from_json($response->decoded_content)->{tournament};
+
 	return $self->{tournament};
 }
 
@@ -583,24 +526,18 @@ sub participant_index
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Make the GET request:
-	$client->GET("/tournaments/$url/participants.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/participants.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, make an object for every participant:
+	# If not, make an object for every participant:
 	my $participants = [];
-	for my $participant(@{from_json($client->responseContent)})
+	for my $participant(@{from_json($response->decoded_content)})
 	{
 		push @{$participants}, WWW::Challonge::Participant->new($participant,
 			$key, $client);
@@ -629,20 +566,14 @@ sub participant_show
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Make the GET request:
-	$client->GET("/tournaments/$url/participants/$participant.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/participants/$participant.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If so, create an object and return it:
 	my $p = WWW::Challonge::Participant->new(from_json($client->responseContent),
@@ -707,13 +638,14 @@ sub participant_create
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Fail if name or challonge_username or email is not provided:
 	unless((defined $args->{name}) || (defined $args->{challonge_username}) ||
 		defined $args->{email})
 	{
-		print STDERR "Error: Name, email or Challonge username are required to
-			create a new participant.\n";
+		carp "Name, email or Challonge username are required to create a new ".
+			"participant.\n";
 		return undef;
 	}
 
@@ -724,23 +656,15 @@ sub participant_create
 	my $params = { api_key => $key, participant => $args };
 
 	# Now we have all the arguments validated, send the POST request:
-	$client->POST("/tournaments/$url/participants.json", to_json($params),
-		{ "Content-Type" => 'application/json' });
+	my $response = $client->request(WWW::Challonge::__json_request(
+		"$HOST/tournaments/$url/participants.json", "POST", $params));
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
-	# If so, create an object and return it:
-	my $p = WWW::Challonge::Participant->new(from_json($client->responseContent),
-		$key, $client);
+	# If not, create an object and return it:
+	my $p = WWW::Challonge::Participant->new(
+		from_json($response->decoded_content), $key, $client);
 	return $p;
 }
 
@@ -768,24 +692,18 @@ sub match_index
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Make the GET request:
-	$client->GET("/tournaments/$url/matches.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/matches.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If so, make an object for every participant:
 	my $matches = [];
-	for my $match(@{from_json($client->responseContent)})
+	for my $match(@{from_json($response->decoded_content)})
 	{
 		push @{$matches}, WWW::Challonge::Match->new($match, $key,
 			$client);
@@ -814,20 +732,14 @@ sub match_show
 	my $key = $self->{key};
 	my $client = $self->{client};
 	my $url = $self->{tournament}->{url};
+	my $HOST = $WWW::Challonge::HOST;
 
 	# Make the GET request:
-	$client->GET("/tournaments/$url/matches/$match.json?api_key=$key");
+	my $response = $client->get(
+		"$HOST/tournaments/$url/matches/$match.json?api_key=$key");
 
-	# Check if it was successful:
-	if($client->responseCode > 300)
-	{
-		my $errors = from_json($client->responseContent)->{errors};
-		for my $error(@{$errors})
-		{
-			print STDERR "Error: $error\n";
-		}
-		return undef;
-	}
+	# Check for any errors:
+	WWW::Challonge::__handle_error $response if($response->is_error);
 
 	# If so, create an object and return it:
 	my $m = WWW::Challonge::Match->new(from_json($client->responseContent),
@@ -845,7 +757,7 @@ that has been successfully destroyed.
 
 sub __is_kill
 {
-	print STDERR "Error: Tournament has been destroyed\n";
+	croak "Tournament has been destroyed";
 	return undef;
 }
 
@@ -863,7 +775,7 @@ sub __args_are_valid
 	# Check it is a hashref:
 	unless(ref $args eq "HASH")
 	{
-		print STDERR "Error: Argument must be a hashref.\n";
+		carp "Argument must be a hashref";
 		return undef;
 	}
 
@@ -920,9 +832,7 @@ sub __args_are_valid
 		{
 			if(length $args->{$arg} > 60)
 			{
-				print STDERR "Error: Name '", $args->{$arg}, " is longer than ",
-					"60 characters";
-				return undef;
+				croak "Name '" . $args->{$arg} . " is longer than 60 characters";
 			}
 		}
 		elsif($arg =~ /^tournament_type$/)
@@ -930,18 +840,15 @@ sub __args_are_valid
 			if($args->{$arg} !~ /^((single|double) elimination)|(round robin)|
 				(swiss)$/i)
 			{
-				print STDERR "Error: Value '", $args->{$arg}, "' is invalid ",
-					"for argument '", $arg, "'\n";
-				return undef;
+				croak "Value '" . $args->{$arg} . "' is invalid for argument '".
+					$arg . "'";
 			}
 		}
 		elsif($arg =~ /^url$/)
 		{
 			if($args->{$arg} !~ /^[a-zA-Z0-9_]*$/)
 			{
-				print STDERR "Error: Value '", $args->{$arg}, "' is not a ",
-					"valid URL.\n";
-				return undef;
+				croak "Value '" . $args->{$arg} . "' is not a valid URL";
 			}
 		}
 		elsif($arg =~ /^ranked_by$/)
@@ -949,9 +856,8 @@ sub __args_are_valid
 			if($args->{$arg} !~ /^((match|game) wins)|
 				(points (scored|difference))|custom/i)
 			{
-				print STDERR "Error: Value '", $args->{$arg}, "' is invalid ",
-					"for argument '", $arg, "'\n";
-				return undef;
+				croak "Value '" . $args->{$arg} . "' is invalid for argument '".
+					$arg . "'";
 			}
 		}
 	}
@@ -961,9 +867,8 @@ sub __args_are_valid
 		# Make sure the argument is an integer:
 		if($args->{$arg} !~ /^\d*$/)
 		{
-			print STDERR "Error: Value '", $args->{$arg}, "' is not a valid ",
-				"integer for argument '", $arg, "'\n";
-			return undef;
+			croak "Value '" . $args->{$arg} . "' is not a valid integer for " .
+				"argument '" . $arg . "'";
 		}
 	}
 	for my $arg(@{$valid_args{decimal}})
@@ -972,9 +877,8 @@ sub __args_are_valid
 		# Make sure the argument is an integer or decimal:
 		if($args->{$arg} !~ /^\d*\.?\d*$/)
 		{
-			print STDERR "Error: Value '", $args->{$arg}, "' is not a valid ",
-				"decimal for argument '", $arg, "'\n";
-			return undef;
+			croak "Value '" . $args->{$arg} . "' is not a valid decimal for " .
+				"argument '" . $arg . "'";
 		}
 		else
 		{
@@ -987,9 +891,8 @@ sub __args_are_valid
 		# Make sure the argument is true or false:
 		if($args->{$arg} !~ /^(true|false)$/i)
 		{
-			print STDERR "Error: Value '", $args->{$arg}, "' is not a valid ",
-				"for argument '", $arg, "'. It should be 'true' or 'false'.\n";
-			return undef;
+			croak "Value '", $args->{$arg}, "' is not valid for argument '" .
+				$arg . "'. It should be 'true' or 'false'";
 		}
 	}
 	for my $arg(@{$valid_args{datetime}})
@@ -998,9 +901,8 @@ sub __args_are_valid
 		# Make sure the argument is a valid datetime:
 #		if($args->{$arg} !~ /^$/)
 #		{
-#			print STDERR "Error: Value '", $args->{$arg}, "' is not a valid ",
-#				"for argument '", $arg, "'. It should be 'true' or 'false'.\n";
-#			return undef;
+#			carp "Value '", $args->{$arg}, "' is not a valid datetime for " .
+#				"argument '" . $arg . "'";
 #		}
 	}
 
@@ -1010,7 +912,7 @@ sub __args_are_valid
 		@{$valid_args{string}},
 		@{$valid_args{integer}},
 		@{$valid_args{decimal}},
-		@{$valid_args{boolean}},
+		@{$valid_args{bool}},
 		@{$valid_args{datetime}}
 	);
 	my $is_valid = 0;
@@ -1024,8 +926,7 @@ sub __args_are_valid
 				last;
 			}
 		}
-		print STDERR "Warning: Ignoring unknown argument '", $arg, "'\n"
-			unless($is_valid);
+		carp "Ignoring unknown argument '" . $arg . "'" unless($is_valid);
 		$is_valid = 0;
 	}
 	return 1;
